program ex1_comp_unistat
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine COMP_UNISTAT
!   in module Stat_Procedures .
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, comp_unistat
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=26, m=20, p=250
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                      :: err1, err2, eps
    real(stnd), dimension(n,m,7)    :: xstat1, xstat2
    real(stnd), dimension(n,m,p)    :: x
!
    integer(i4b) :: i
!
    logical(lgl) :: first, last
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of comp_unistat'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x .
!
    call random_number( x )
!
!   COMPUTE THE STATISTICS OF x FOR THE p OBSERVATIONS .
!
    first = true
    last  = true
!
    call comp_unistat( x(:n,:m,:p), first, last, xstat1(:n,:m,:7) )
!
!   ON EXIT, WHEN last=true, xstat1 CONTAINS THE FOLLOWING
!   STATISTICS ON ALL VARIABLES :
!
!        xstat1(:,:,1) CONTAINS THE MEAN VALUES.
!        xstat1(:,:,2) CONTAINS THE VARIANCES.
!        xstat1(:,:,3) CONTAINS THE STANDARD DEVIATIONS.
!        xstat1(:,:,4) CONTAINS THE COEFFICIENTS OF SKEWNESS.
!        xstat1(:,:,5) CONTAINS THE COEFFICIENTS OF KURTOSIS.
!        xstat1(:,:,6) CONTAINS THE MINIMA.
!        xstat1(:,:,7) CONTAINS THE MAXIMA.
!
!   COMPUTE THE STATISTICS OF x, ITERATIVELY FOR THE p OBSERVATIONS  .
!
    do i = 1, p
!
        first = i==1
        last  = i==p
!
        call comp_unistat( x(:n,:m,i:i), first, last, xstat2(:n,:m,:7) )
    end do
!
!   CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
    err1 = maxval( abs( (xstat2(:,:,1:3)-xstat1(:,:,1:3))/xstat1(:,:,1:3) ) )
    err2 = maxval( abs( xstat2(:,:,4:7)-xstat1(:,:,4:7) ) )
!
    if ( max(err1, err2)<=eps ) then
        write (prtunit,*) 'Example 1 of COMP_UNISTAT is correct'
    else
        write (prtunit,*) 'Example 1 of COMP_UNISTAT is incorrect'
    end if
!
!
! END OF PROGRAM ex1_comp_unistat
! ===============================
!
end program ex1_comp_unistat
