program ex1_daynum_to_ymd
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of function YMD_TO_DAYNUM and
!   subroutine DAYNUM_TO_YMD in module Time_Procedures .
!                                                                              
!   See also program ex1_ymd_to_daynum.f90 .   
!                                                                              
! LATEST REVISION : 26/06/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, ymd_to_daynum, daynum_to_ymd
!   
!
! STRONG TYPING IMPOSED
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT.
!
    integer(i4b), parameter :: prtunit=6
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    integer(i4b) :: iyr, imon, iday, iyr2, imon2, iday2, idaynum
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of daynum_to_ymd'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A DATE.
!
    iyr  = 1902
    imon = 11
    iday = 15
!
!   CONVERTS GREGORIAN YEAR (iyr), MONTH (imon) AND DAY (iday) TO JULIAN DAY
!   NUMBER (idaynum).
!
    idaynum = ymd_to_daynum( iyr, imon, iday )
!
!   FUNCTION ymd_to_daynum CONVERTS THE THREE INTEGERS iyr, imon AND iday STANDING FOR
!   YEAR, MONTH, DAY IN THE GREGORIAN CALENDAR PROMULGATED BY GREGORY XIII ON
!   FRIDAY, 15 OCTOBER 1582, IN THE CORRESPONDING JULIAN DAY NUMBER STARTING
!   WITH ymd_to_daynum=1 ON FRIDAY, 15 OCTOBER 1582.
!
!   NOTE THE GREGORIAN CALENDAR WAS ADOPTED IN OCT. 15, 1582, AND HENCE
!   THIS FUNCTION WILL NOT WORK PROPERLY FOR DATES EARLY THAN 10-15-1582.
!   
!   CONVERTS A JULIAN DAY NUMBER (idaynum) TO GREGORIAN YEAR (iyr2), MONTH (imon2)
!   AND DAY (iday2).
!
    call daynum_to_ymd( idaynum, iyr2, imon2, iday2 )
!
!   SUBROUTINE daynum_to_ymd CONVERTS THE INTEGER idaynum TO THREE INTEGERS iyr2, imon2 AND
!   iday2 STANDING FOR YEAR, MONTH, DAY IN THE GREGORIAN CALENDAR PROMULGATED BY 
!   GREGORY XIII STARTING WITH idaynum=1 ON FRIDAY, 15 OCTOBER 1582.
!
!   TO KEEP POPE GREGORY'S CALENDAR SYNCHRONIZED WITH THE SEASONS FOR THE NEXT 
!   16000 YEARS OR SO, A SMALL CORRECTION HAS BEEN INTRODUCED; MILLENNIAL YEARS
!   DIVISIBLE BY 4000 ARE NOT CONSIDERED LEAP-YEARS.
!
!   NOTE THE GREGORIAN CALENDAR WAS ADOPTED IN OCT. 15, 1582, AND HENCE
!   THIS SUBROUTINE WILL NOT WORK PROPERLY FOR DATES EARLY THAN 10-15-1582,
!   EG. IF idaynum < 1.
!
!   CHECK THE COMPUTATIONS.
!
    if ( iyr==iyr2 .and. imon==imon2 .and. iday==iday2  ) then
        write (prtunit,*) 'Example 1 of DAYNUM_TO_YMD is correct'
    else
        write (prtunit,*) 'Example 1 of DAYNUM_TO_YMD is incorrect'
    end if
!
!
! END OF PROGRAM ex1_daynum_to_ymd
! ================================
!
end program ex1_daynum_to_ymd
