program ex1_fftxy
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine FFTXY
!   in module FFT_Procedures.
!                                                                              
! LATEST REVISION : 15/06/2016
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, one, true, false, init_fft, fft, fftxy, &
                         end_fft, merror, allocate_error
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=1000024
!
    character(len=*), parameter :: name_proc='Example 1 of fftxy'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    integer :: iok, istart, iend, irate, imax, itime
!
    real(stnd)                            :: errx, erry, eps, elapsed_time
    real(stnd), dimension(:), allocatable :: x, y, x2, y2
!
    complex(stnd), dimension(:), allocatable :: fftx, ffty
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   EXAMPLE 1 : FORWARD FFTS OF TWO REAL SEQUENCES.
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon( errx ) )
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( x(n), y(n), x2(n), y2(n), fftx(n), ffty(n), stat=iok )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   GENERATE TWO RANDOM NUMBER REAL SEQUENCES.
!
    call random_number( x(:n) )
    call random_number( y(:n) )
!
!   SAVE THE REAL SEQUENCES.
!
    x2(:n) = x(:n)
    y2(:n) = y(:n)
!
!   INITIALIZE THE FFT SUBROUTINE.
!
    call init_fft( n )
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count_rate=irate, count_max=imax  )
    call system_clock( count=istart )
!
!   TRANSFORM AND THEN INVERT THE TWO REAL SEQUENCES BACK.
!
    call fftxy( x(:n), y(:n), fftx(:n), ffty(:n)  )
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    itime = iend - istart
    if ( iend<istart ) then
        itime = itime + imax   
    end if
!
    elapsed_time = real( itime, stnd )/real( irate, stnd )
!
    call fft( fftx(:n), forward=false  )
    x(:n) = real( fftx(:n) )
!
    call fft( ffty(:n), forward=false )
    y(:n) = real( ffty(:n) )
!
    call end_fft()
!
!   CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
    errx = maxval(abs(x2(:n)-x(:n)))/maxval(abs(x2(:n)))
    erry = maxval(abs(y2(:n)-y(:n)))/maxval(abs(y2(:n)))
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( x, y, x2, y2, fftx, ffty )
!
!   PRINT RESULT OF THE TEST.
!
    if (max(errx,erry)<=eps  ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
    write (prtunit,*)
    write (*,'(a,i8,a,0pd12.4,a)')    &
      'The elapsed time for computing the forward FFTs of two real sequences of size ', &
       n, ' is', elapsed_time, ' seconds'
!
!
! END OF PROGRAM ex1_fftxy
! ========================
!
end program ex1_fftxy
