program ex1_hwfilter
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine HWFILTER
!   in module Time_Series_Procedures .
!                                                                              
! LATEST REVISION : 30/03/2007
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, one, hwfilter
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT, n IS THE LENGTH OF THE TIME SERIES.
!
    integer(i4b), parameter :: prtunit=6, n=5000
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)               :: err, win
    real(stnd), dimension(n) :: y, y2, y3
!
    integer(i4b) :: minp, maxp
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of hwfilter'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM REAL NUMBER SEQUENCE OF LENGTH n .
!
    call random_number( y(:n) )
!
!   SAVE THE REAL RANDOM NUMBER ARRAY.
!
    y2(:n) = y(:n)
    y3(:n) = y(:n)
!
    minp  = 18_i4b
    maxp  = 96_i4b
!
!   BY DEFAULT, HAMMMING WINDOW FILTERING IS USED (i.e. win=0.54).
!   SET win=0.5 for HANNING WINDOW OR win=1 FOR RECTANGULAR WINDOW.
!   IN ANY CASE, win MUST BE GREATER OR EQUAL TO 0.5 AND LESS OR EQUAL TO 1.
!
    win = one
!
!   hwfilter FILTERS A TIME SERIES (THE ARGUMENT VEC) IN THE FREQUENCY BAND
!   LIMITED BY PERIODS PL AND PH BY WINDOWED FILTERING (PL AND PH ARE EXPRESSED IN NUMBER OF
!   POINTS, i.e. PL=18 AND PH=96 SELECTS PERIODS BETWEEN 1.5 YRS AND 8 YRS FOR MONTHLY DATA).
!
!   FILTER THE TIME SERIES, KEEP THE PERIODS BETWEEN minp AND maxp .
!
    call hwfilter( VEC=y2(:n), PL=minp, PH=maxp, WIN=win )
!
!   SETTING PH<PL IS ALLOWED AND PERFORMS BAND REJECTION OF PERIODS BETWEEN PH AND PL.
!
!   FILTER THE TIME SERIES, KEEP THE PERIODS OUTSIDE THE BAND BETWEEN minp AND maxp .
!
    call hwfilter( VEC=y3(:n), PL=maxp, PH=minp, WIN=win )
!
!   NOW RECONSTRUCT THE ORIGINAL TIME SERIES FROM THE TWO FILTERED TIME SERIES.
!
    y2(:n) = y2(:n) + y3(:n)
!
!   TEST THE ACCURACY OF THE RECONSTRUCTION.
!
    err = maxval(abs(y(:n)-y2(:n)))/maxval(abs(y(:n)))
!
    if ( err<=sqrt(epsilon(err))  ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex1_hwfilter
! ===========================
!
end program ex1_hwfilter
