program ex1_inv
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of fonction INV
!   in module Lin_Procedures . 
!                                                                              
! LATEST REVISION : 18/06/2017
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, true, false, is_nan, zero, one, inv, &
                         norm, merror, allocate_error
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=4000
!
    character(len=*), parameter :: name_proc='Example 1 of inv'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err, eps, elapsed_time
    real(stnd), dimension(:,:), allocatable :: a, ainv, res
!
    integer(i4b) :: j
    integer      :: iok, istart, iend, irate, imax, itime
!
    logical(lgl) :: do_test, failure
!
!
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   EXAMPLE 1 : COMPUTE THE INVERSE OF A REAL n-BY-n MATRIX.
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon( err ) )
    err = zero
!
    do_test = true
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( a(n,n), ainv(n,n), stat=iok )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   GENERATE A RANDOM DATA MATRIX a .
!
    call random_number( a )
!
!   START TIMING THE COMPUTATIONS.
!
    call system_clock( count_rate=irate, count_max=imax  )
    call system_clock( count=istart )
!
!   COMPUTE THE MATRIX INVERSE WITH FUNCTION inv.
!
    ainv = inv( a ) 
!
!   STOP THE TIMER.
!
    call system_clock( count=iend )
!
    itime = iend - istart
    if ( iend<istart ) then
        itime = itime + imax   
    end if
!
    elapsed_time = real( itime, stnd )/real( irate, stnd )
!
!   CHECK IF THE MATRIX IS SINGULAR.
!
    failure = is_nan( ainv )
!
    if ( failure ) then
!
!       ANORMAL EXIT FROM inv FUNCTION, PRINT A WARNING.
!
        write (prtunit,*) 'Error in exit of INV function, input matrix is singular'
        write (prtunit,*)
!
    else if ( do_test ) then
!
!       ALLOCATE WORK ARRAY.
!
        allocate( res(n,n), stat=iok )
!
        if ( iok/=0 ) then
            call merror( name_proc//allocate_error )
        end if
!
!       COMPUTE a TIMES ITS INVERSE - IDENTITY.
!
        res = matmul( a, ainv )
!
        do j = 1_i4b, n
            res(j,j) = res(j,j) - one
        end do
!
        err = norm( res(:n,:n) ) /( real(n,stnd)*norm(a(:n,:n)) )
!
!       DEALLOCATE WORK ARRAY.
!
        deallocate( res )
!
    end if
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( ainv, a )
!
!   CHECK THE RESULTS FOR SMALL RESIDUALS.
!
    if ( err<=eps .and. .not.failure ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
    write (prtunit,*)
    write (*,'(a,i5,a,i5,a,0pd12.4,a)')    &
      'The elapsed time for computing the inverse of a real matrix of size ', &
       n, ' by ', n, ' is', elapsed_time, ' seconds'
!
!
! END OF PROGRAM ex1_inv
! ======================
!
end program ex1_inv
