program ex1_probn2
!
!
! Purpose
! =======
!
!   This program is intended to illustrate the use of functions PROBN2 and PINVN2
!   in module Prob_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 15/02/2013
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, extd, false, probn2, pinvn2
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=5000, m=10000
!
    real(extd), parameter :: eps = 1.0e-6_extd
!
    character(len=*), parameter :: name_proc='Example 1 of probn2'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(extd), dimension(n,m) :: p, p2, x
    real(extd)                 :: err
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM PROBABILITY MATRIX p .
!
    call random_number( p(:n,:m) )
!
!   COMPUTE THE NORMAL DEVIATES CORRESPONDING TO LOWER TAIL AREAS OF P .
!
    x(:n,:m) = pinvn2( p(:n,:m) ) 
!
!   RECOMPUTE THE PROBABILITIES FROM THE NORMAL DEVIATES .
!
    p2(:n,:m) = probn2( x(:n,:m), upper=false )
!
!   CHECK THAT p AND p2 AGREE.
!
    err = maxval( abs( p(:n,:m) - p2(:n,:m) ) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=eps ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex1_probn2
! =========================
!
end program ex1_probn2
