program ex1_probq3
!
!
! Purpose
! =======
!
!   This program is intended to illustrate the use of functions PROBQ3, PINVQ2
!   in module Prob_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 26/02/2013
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, false, probq3, pinvq2
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=5000, m=10000
!
    real(stnd), parameter :: eps = 1.0e-3_stnd 
!
    character(len=*), parameter :: name_proc='Example 1 of probq3'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), dimension(n,m) :: p, p2, x2
    real(stnd)                 :: err, df
!
    integer(i4b) :: i, j
!
    logical(lgl) :: upper
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE THE DEGREES OF FREEDOM OF CHI-SQUARED DISTRIBUTION.
!   DF IS NOT NECESSARILY AN INTEGER.
!
    df = 50.5_stnd
!
!   GENERATE A RANDOM PROBABILITY MATRIX p(:m,:n) .
!
    call random_number( p(:n,:m) )
!
!   COMPUTE THE QUANTILES x2(:m,:n) OF CHI-SQUARED DISTRIBUTION WITH df DEGREES OF FREEDOM
!   CORRESPONDING TO LOWER TAIL AREAS OF p(:m,:n) .
!
    x2(:n,:m) = pinvq2( p(:n,:m), df )
!
!   RECOMPUTE THE PROBABILITIES FROM THE QUANTILES
!   WITH probq3 FUNCTION.
!
    upper = false
!
    p2(:n,:m) = probq3( x2(:n,:m), df, upper=upper )
!
!   CHECK THAT p(:n,:m) AND p2(:n,:m) AGREE.
!
    err = maxval( abs( p(:n,:m) - p2(:n,:m) ) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=eps ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex1_probq3
! =========================
!
end program ex1_probq3
