program ex1_probstudent
!
!
! Purpose
! =======
!
!   This program is intended to illustrate the use of functions PROBSTUDENT, PINVSTUDENT
!   in module Prob_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 26/02/2013
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, false, probstudent, pinvstudent
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=5000, m=10000
!
    real(stnd), parameter :: eps = 1.0e-4_stnd 
!
    character(len=*), parameter :: name_proc='Example 1 of probstudent'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), dimension(n,m) :: p, p2, t
    real(stnd)                 :: err, df
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE THE DEGREES OF FREEDOM OF STUDENT T-DISTRIBUTION .
!
    df = 50._stnd
!
!   GENERATE A RANDOM PROBABILITY MATRIX p(:n,:m) .
!
    call random_number( p(:n,:m) )
!
!   COMPUTE THE TWO-TAIL QUANTILES t OF STUDENT T-DISTRIBUTION 
!   WITH df DEGREES OF FREEDOM CORRESPONDING TO AREAS OF p(:n,:m) .
!
    t(:n,:m) = pinvstudent( p(:n,:m), df )
!
!   RECOMPUTE THE PROBABILITIES FROM THE QUANTILES
!   WITH probstudent FUNCTION.
!
    p2(:n,:m) = probstudent( t(:n,:m), df )
!
!   CHECK THAT p AND p2 AGREE.
!
    err = maxval( abs( p(:n,:m) - p2(:n,:m) ) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=eps ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex1_probstudent
! ==============================
!
end program ex1_probstudent
