program ex1_solve_llsq
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of fonction SOLVE_LLSQ
!   in module LLSQ_Procedures . 
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, solve_llsq
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=100, m=1000
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                 :: err
    real(stnd), dimension(m)   :: b, res
    real(stnd), dimension(n)   :: x
    real(stnd), dimension(m,n) :: a
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 1 of solve_llsq'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   EXAMPLE 1 : LINEAR LEAST SQUARES SYSTEM AND ONE RIGHT HAND-SIDE.
!
!
!   GENERATE A RANDOM COEFFICIENT MATRIX a .
!
    call random_number( a )
!
!   GENERATE A RANDOM RIGHT HAND-SIDE vector b .
!
    call random_number( b )
!
!   COMPUTE SOLUTION VECTOR FOR LINEAR LEAST SQUARES SYSTEM
!
!             a*x(:)=b(:) .
!
    x = solve_llsq( a, b )
!
!   CHECK THAT THE RESIDUALS ARE ORTHOGONAL TO THE COLUMN VECTORS OF a .
!
    res = b - matmul( a, x )
    err = sum(abs(matmul(res,a)) )/ sum( abs(a) )
!
    if ( err<=sqrt(epsilon(err)) ) then
        write (prtunit,*) 'Example 1 of SOLVE_LLSQ is correct'
    else
        write (prtunit,*) 'Example 1 of SOLVE_LLSQ is incorrect'
    end if
!
!
! END OF PROGRAM ex1_solve_llsq
! =============================
!
end program ex1_solve_llsq
