program ex2_bd_singval2
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine BD_SINGVAL2
!   in module SVD_Procedures .
!                                                                              
!                                                                              
! Further Details
! ===============
!
!    The program also shows the use of subroutines BD_CMP, APPLY_Q_BD, APPLY_P_BD
!    in module SVD_Procedures.
!
! LATEST REVISION : 22/07/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, true, false, zero, bd_inviter,     &
                         bd_cmp, bd_singval2, apply_q_bd, apply_p_bd,       &
                         merror, allocate_error, norm, c50
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=1000, m=500, mn=min(m,n), ls=20
!   
    real(stnd), parameter  :: fudge=c50
!
    character(len=*), parameter :: name_proc='Example 2 of bd_singval2'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err, eps
    real(stnd), dimension(n,m)              :: a, a2
    real(stnd), dimension(:,:), allocatable :: leftvec, rightvec
    real(stnd), dimension(mn)               :: s, d, e, tauq, taup
!
    integer(i4b) :: maxiter=2, nsing
    integer      :: iok
!
    logical(lgl) :: failure, bd_is_upper
!   
    character    :: sort='d'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = fudge*epsilon( err )
!
    err = zero
!
!   GENERATE A RANDOM DATA MATRIX a.
!
    call random_number( a )
!
!   SAVE RANDOM DATA MATRIX a .
!
    a2(:n,:m) = a(:n,:m)
!
!   REDUCE a TO LOWER OR UPPER BIDIAGONAL FORM.
!
    call bd_cmp( a, d, e, tauq, taup )
!
!   COMPUTE THE FIRST ls SINGULAR VALUES OF BIDIAGONAL FORM OF a BY A BISECTION METHOD.
!
    call bd_singval2( d, e, nsing, s, failure, sort=sort, vector=true, ls=ls  )
!
    if ( .not. failure .and. nsing>0 ) then
!
!       ALLOCATE WORK VARIABLES NEEDED TO STORE THE SINGULAR VECTORS.
!
        allocate( leftvec(n,nsing),    &
                  rightvec(m,nsing),   &
                  stat = iok    )
!
        if ( iok/=0 ) then
            call merror( name_proc//allocate_error )
        end if
!
        bd_is_upper = n>=m
!
!       COMPUTE THE FIRST nsing SINGULAR VECTORS OF a BY maxiter
!       INVERSE ITERATIONS ON THE INTERMEDIATE BIDIAGONAL MATRIX
!       d_e AND BACK-TRANSFORMATION.
!
        call bd_inviter( bd_is_upper, d, e, s(:nsing), leftvec(:mn,:nsing), rightvec(:mn,:nsing),  &
                         failure, maxiter=maxiter )
!
!       COMPUTE SINGULAR VECTORS OF a BY BACK-TRANSFORMATION.
!
        if ( bd_is_upper ) then
            leftvec(mn+1_i4b:n,:nsing) = zero
        else
            rightvec(mn+1_i4b:m,:nsing) = zero
        end if
!
!       GENERATE LEFT SINGULAR VECTORS OF a IN leftvec BY BACK-TRANSFORMATION.
!
        call apply_q_bd( a, tauq, leftvec,  left=true, trans=false )
!
!       GENERATE RIGHT SINGULAR VECTORS OF a IN rightvec BY BACK-TRANSFORMATION.
!
        call apply_p_bd( a, taup, rightvec, left=true, trans=false )
!
!       CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*rightvec - leftvec*s(:nsing),
!       WHERE leftvec AND rightvec ARE THE LEFT AND RIGHT SINGULAR VECTORS.
!
        err =  norm( matmul(a2,rightvec) - leftvec*spread(s(:nsing),dim=1,ncopies=n) )/( norm( a2 )*real(mn,stnd) )
!
!       DEALLOCATE WORK ARRAYS.
!
        deallocate( leftvec,  rightvec  )
!
    end if
!
    if ( err<=eps .and. .not.failure ) then
        write (prtunit,*) 'Example 1 of BD_SINGVAL2 is correct'
    else
        write (prtunit,*) 'Example 1 of BD_SINGVAL2 is incorrect'
    end if
!
!
! END OF PROGRAM ex2_bd_singval2
! ==============================
!
end program ex2_bd_singval2
