program ex2_bd_svd
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine BD_SVD
!   in module SVD_Procedures .
!                                                                              
!                                                     
! Further Details
! ===============
!
!    The program also shows the use of subroutine BD_INVITER in module SVD_Procedures.
!
! LATEST REVISION : 28/09/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, true, false, zero, half, one, c50, &
                         bd_inviter, bd_svd, unit_matrix, norm
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=500, nsing=10
!   
    real(stnd), parameter  :: fudge=c50
!
    character(len=*), parameter :: name_proc='Example 2 of bd_svd'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                              :: err1, err2, err3, err, eps
    real(stnd), dimension(n)                :: diag, sup, sup2, singval
    real(stnd), dimension(n,nsing)          :: leftvec, rightvec
    real(stnd), allocatable, dimension(:,:) :: a, a2, resid
!
    integer      :: iok
    integer(i4b) :: maxiter=2
!
    logical(lgl) :: failure, a_is_upper, do_test
!   
    character    :: sort='d'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = fudge*epsilon( err )
!
    err = zero
    do_test = true
!
!   GENERATE AN UPPER BIDIAGONAL TOEPLITZ MATRIX a.
!   THE DIAGONAL ELEMENTS ARE STORED IN diag .
!   THE OFF-DIAGONAL ELEMENTS ARE STORED IN sup .
!
    a_is_upper   = true
    diag(:n)     = half
    sup(1_i4b)   = zero
    sup(2_i4b:n) = one
!
!   MAKE A COPY OF THE BIDIAGONAL MATRIX.
!
    singval(:n) = diag(:n)
    sup2(:n)    = sup(:n)
!
!    COMPUTE SINGULAR VALUES OF BIDIAGONAL MATRIX a .
!
     call bd_svd( a_is_upper, singval(:n), sup2(:n), failure, sort=sort  )
!
     if ( .not. failure ) then
!
!       COMPUTE THE FIRST nsing SINGULAR VECTORS OF a BY maxiter INVERSE ITERATIONS.
!
        call bd_inviter( a_is_upper, diag(:n), sup(:n), singval(:nsing), leftvec(:n,:nsing), rightvec(:n,:nsing),  &
                         failure, maxiter=maxiter )
!
        if ( do_test ) then
!
            allocate( a(nsing,nsing), a2(nsing,nsing), resid(n,nsing), stat=iok )
!
            if ( iok/=0 ) then
                write (prtunit,*)  'Problem in attempt to allocate memory !'
                stop
            end if
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*rightvec - leftvec*singval(:nsing),
!           WHERE leftvec AND rightvec ARE THE LEFT AND RIGHT SINGULAR VECTORS.
!
            if ( a_is_upper ) then
!
                resid(:n,:nsing) = spread( diag(:n),        dim=2, ncopies=nsing )*rightvec                 + &
                                   eoshift( spread(sup(:n), dim=2,ncopies=nsing)*rightvec, shift=1 )        - &
                                   spread( singval(:nsing), dim=1, ncopies=n )*leftvec 

            else
!
                resid(:n,:nsing) = spread( diag(:n),        dim=2, ncopies=nsing )*leftvec                   + &
                                   eoshift( spread(sup(:n), dim=2,ncopies=nsing)*leftvec, shift=1 )          - &
                                   spread( singval(:nsing), dim=1, ncopies=n )*rightvec
!
            end if
!
            err1 = norm(resid)/(sum( singval(:n) )*real(n,stnd) )
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - u**(t)*u
!           WHERE u are LEFT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX a.
!
            call unit_matrix( a )
!
            a2 = a - matmul( transpose( leftvec ), leftvec )
            err2 = norm(a2)/real(n,stnd)
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION I - v**(t)*v
!           WHERE v are RIGHT SINGULAR VECTORS OF THE BIDIAGONAL MATRIX a.
!
            a2 = a - matmul( transpose( rightvec ), rightvec )
            err3 = norm(a2)/real(n,stnd)
!
            err = max( err1, err2, err3 )
!
            deallocate( a, a2, resid )
!
        end if
!
    end if
!
    if ( err<=eps .and. .not.failure ) then
        write (prtunit,*) 'Example 2 of BD_SVD is correct'
    else
        write (prtunit,*) 'Example 2 of BD_SVD is incorrect'
    end if
!
!
! END OF PROGRAM ex2_bd_svd
! =========================
!
end program ex2_bd_svd
