program ex2_comp_cor_miss
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine COMP_COR_MISS
!   in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, comp_cor_miss
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT.
!
    integer(i4b), parameter :: prtunit=6, n=26, m=20, p=50
!
! miss IS THE MISSING INDICATOR.
!
    real(stnd), parameter :: miss=-999.99_stnd
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                    :: err_xstat, err_ystat, err_cor, eps
    real(stnd), dimension(n,m,4)  :: xycor1, xycor2
    real(stnd), dimension(m,4)    :: ystat1, ystat2
    real(stnd), dimension(n,4)    :: xstat1, xstat2
    real(stnd), dimension(n,p)    :: x
    real(stnd), dimension(m,p)    :: y
!
    integer(i4b) :: i
!
    logical(lgl) :: first, last
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 2 of comp_cor_miss'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x WITH MISSING VALUES.
!
    call random_number( x(:n,:p) )
    where ( x(:n,:p)<=0.05_stnd ) x(:n,:p) = miss
!
!   GENERATE A RANDOM OBSERVATION ARRAY y WITH MISSING VALUES.
!
    call random_number( y(:m,:p) )
    where ( y(:m,:p)<=0.05_stnd ) y(:m,:p) = miss
!
!   COMPUTE THE CORRELATIONS BETWEEN x AND y
!   FOR THE p OBSERVATIONS .
!
    first = true
    last  = true
    call comp_cor_miss( x(:n,:p), y(:m,:p), first, last, xstat1(:n,:4),               &
                        ystat1(:m,:4), xycor1(:n,:m,:4), xymiss=miss )
!
!   ON EXIT OF comp_cor_miss WHEN last=true :
!
!      xstat1(i,1) CONTAINS THE MEAN VALUES OF THE ARRAY SECTION x(i,:p).
!
!      xstat1(i,2) CONTAINS THE VARIANCES OF THE ARRAY SECTION x(i,:p).
!
!      xstat1(i,3) CONTAINS THE NUMBER OF NON-MISSING OBSERVATIONS
!                  IN THE ARRAY SECTION x(i,:p).
!
!      ystat1(j,1) CONTAINS THE MEAN VALUE OF THE ARRAY SECTION y(j,:p).
!
!      ystat1(j,2) CONTAINS THE VARIANCE OF THE ARRAY SECTION y(j,:p).
!
!      ystat1(j,3) CONTAINS THE NUMBER OF NON-MISSING OBSERVATIONS
!                  IN THE ARRAY SECTION y(j,:p).
!
!      xycor1(i,j,1) CONTAINS THE CORRELATION COEFFICIENT BETWEEN x(i,:p) AND y(j,:p)
!                    COMPUTED WITH THE ABOVE UNIVARIATE STATISTICS.
!
!      xycor1(i,j,2) CONTAINS THE INCIDENCE VALUE BETWEEN x(i,:p) AND y(j,:p).
!                    xycor1(i,j,2) INDICATES THE NUMBER OF VALID PAIRS OF OBSERVATIONS
!                    WHICH WHERE USED IN THE CALCULATION OF xycor1(i,j,1) .
!
!   xstat1(:,4), ystat1(:,4) AND xycor1(:,:,3:4) ARE USED AS WORKSPACE AND CONTAIN NO USEFUL
!   INFORMATION ON OUTPUT OF comp_cor_miss.
!
!   COMPUTE CORRELATIONS BETWEEN x AND y,
!   ITERATIVELY  FOR THE p OBSERVATIONS .
!
    do i = 1, p
!
        first = i==1
        last  = i==p
!
        call comp_cor_miss( x(:n,i:i), y(:m,i:i), first, last, xstat2(:n,:4),       &
                            ystat2(:m,:4), xycor2(:n,:m,:4), xymiss=miss )
    end do
!
!   CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
    err_xstat = maxval( abs( ( xstat1(:n,:3)-xstat2(:n,:3))/xstat1(:n,:3) ) )
    err_ystat = maxval( abs( ( ystat1(:m,:3)-ystat2(:m,:3))/ystat1(:m,:3) ) )
    err_cor   = maxval( abs( xycor1(:n,:m,:2)-xycor2(:n,:m,:2) ) )
!
    if ( max(err_xstat, err_ystat, err_cor )<=eps ) then
        write (prtunit,*) 'Example 2 of COMP_COR_MISS is correct'
    else
        write (prtunit,*) 'Example 2 of COMP_COR_MISS is incorrect'
    end if
!
!
!
! END OF PROGRAM ex2_comp_cor_miss
! ================================
!
end program ex2_comp_cor_miss
