program ex2_comp_cormat
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine COMP_CORMAT
!   in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, false, comp_cormat
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=20, p=500, n=(m*(m+1))/2
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                       :: err_mean, err_std, err_cor, eps, xn
    real(stnd), dimension(n)         :: corp1, corp2
    real(stnd), dimension(m,p)       :: x
    real(stnd), dimension(m)         :: mean1, mean2, std1, std2
!
    integer(i4b) :: i
!
    logical(lgl) :: first, last, cov
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 2 of comp_cormat'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x .
!
    call random_number( x )
!
    cov = false
!
!   COMPUTE THE MEANS, STANDARD-DEVIATIONS AND CORRELATION MATRIX
!   OF x FOR THE p OBSERVATIONS .
!
    first = true
    last  = true
    call comp_cormat( x(:m,:p), first, last, mean1(:m), corp1(:n), xn,    &
                      xstd=std1(:m), cov=cov )
!
!   ON EXIT, WHEN last=true :
!
!     mean1(:m) CONTAINS THE VARIABLE MEANS COMPUTED FROM ALL OBSERVATIONS 
!     IN THE DATA MATRIX x.
!
!     THE UPPER TRIANGLE OF THE SYMMETRIC CORRELATION OR VARIANCE-COVARIANCE MATRIX cor,
!     AS CONTROLLED BY THE cov ARGUMENT, IS PACKED COLUMNWISE IN THE LINEAR ARRAY corp1.
!     MORE PRECISELY, THE J-TH COLUMN  OF cor IS STORED IN THE ARRAY CORP1 AS FOLLOWS:
!
!            corp1(i + (j-1)*j/2) = cor(i,j) for 1<=i<=j;
!
!     xn INDICATES THE NUMBERS OF OBSERVATIONS WHICH WERE 
!     USED IN THE CALCULATION OF corp1.
!
!     IF THE OPTIONAL ARGUMENT xstd IS PRESENT, xstd CONTAINS THE VARIABLE 
!     STANDARD-DEVIATIONS.
!
!   COMPUTE THE MEANS, STANDARD-DEVIATIONS AND CORRELATION MATRIX
!   OF x, ITERATIVELY  FOR THE p OBSERVATIONS .
!
    do i = 1, p
!
        first = i==1
        last =  i==p
!
        call comp_cormat( x(:m,i:i), first, last, mean2(:m), corp2(:n), xn,    &
                         xstd=std2(:m), cov=cov )
    end do
!
!   CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
    err_mean = maxval( abs( ( mean1-mean2)/mean1 ) )
    err_std  = maxval( abs( ( std1-std2)/std1    ) )
    err_cor  = maxval( abs( corp1-corp2 ) )
!
    if ( max(err_mean, err_std, err_cor )<=eps ) then
        write (prtunit,*) 'Example 2 of COMP_CORMAT is correct'
    else
        write (prtunit,*) 'Example 2 of COMP_CORMAT is incorrect'
    end if
!
!
!
! END OF PROGRAM ex2_comp_cormat
! ==============================
!
end program ex2_comp_cormat
