program ex2_comp_mvs
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine COMP_MVS
!   in module Stat_Procedures .
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, comp_mvs
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! xmiss IS THE MISSING INDICATOR
!
    real(stnd), parameter  :: xmiss=-999.99_stnd
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=26, m=20, p=250
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                      :: err_mean, err_std, err_var, eps
    real(stnd), dimension(n,m)      :: xmean1, xmean2, xstd1, xstd2, xvar1, xvar2
    real(stnd), dimension(n,m,p)    :: x
!
    integer(i4b)                  :: i
    integer(i4b), dimension(n,m)  :: xnobs1, xnobs2
!
    logical(lgl) :: first, last
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='Example 2 of comp_mvs'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
    eps = sqrt( epsilon(err_mean) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x WITH MISSING VALUES .
!
    call random_number( x )
!
    where( x(:,:,:)<=0.25_stnd )  x(:,:,:) = xmiss
!
!   COMPUTE THE MEANS, VARIANCES, STANDARD-DEVIATIONS AND NUMBER OF OBSERVATIONS 
!   OF x FOR THE p OBSERVATIONS .
!
    first = true
    last  = true
!
    call comp_mvs( x(:,:,:), first, last, xmean1(:,:), xvar1(:,:), xstd1(:,:),  &
                   xmiss=xmiss, xnobs=xnobs1(:,:) )
!
!   COMPUTE THE MEANS, VARIANCES, STANDARD-DEVIATIONS AND NUMBER OF OBSERVATIONS OF x
!   ITERATIVELY FOR THE p OBSERVATIONS  .
!
    do i = 1, p
!
        first = i==1
        last  = i==p
!
        call comp_mvs( x(:,:,i:i), first, last, xmean2(:,:), xvar2(:,:), xstd2(:,:),  &
                       xmiss=xmiss, xnobs=xnobs2(:,:)  )
    end do
!
!   CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
    err_mean = maxval( abs( ( xmean1-xmean2)/xmean1 ) )
    err_var  = maxval( abs( ( xvar1-xvar2)/xvar1    ) )
    err_std  = maxval( abs( ( xstd1-xstd2)/xstd1    ) )
!
    if ( max(err_mean, err_var, err_std )<=eps .and. all( xnobs2(:,:)==xnobs1(:,:) ) ) then
        write (prtunit,*) 'Example 2 of COMP_MVS is correct'
    else
        write (prtunit,*) 'Example 2 of COMP_MVS is incorrect'
    end if
!
!
! END OF PROGRAM ex2_comp_mvs
! ===========================
!
end program ex2_comp_mvs
