program ex2_eigval_cmp
!
!
! Purpose
! =======
!
!   This program is intended to demonstrate the use of subroutine EIGVAL_CMP
!   in module Eig_Procedures .
!                                                                              
! LATEST REVISION : 06/07/2006
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, true, triangle, trid_inviter, eigval_cmp
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=1000, p=(n*(n+1))/2, m=10
!
    character(len=*), parameter :: name_proc='Example 2 of eigval_cmp'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                   :: vec(p), d(n), d_e(n,2), eigvec(n,m), err
    real(stnd), dimension(n,n)   :: a
!
    integer(i4b)     :: maxiter=4
!
    logical(lgl)     :: failure, failure2, upper=true
!   
    character        :: sort='d'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE A RANDOM DATA MATRIX AND FROM IT
!   A SELF-ADJOINT MATRIX a .
!
    call random_number( a )
    a = a + transpose( a ) 
!
!   MAKE A COPY OF THE SELF-ADJOINT MATRIX IN PACKED FORM.
!
    vec(:p) = pack( a, mask=triangle( upper, n, n, extra=1_i4b) )
!
!   COMPUTE ALL EIGENVALUES OF THE SELF-ADJOINT MATRIX a IN PACKED FORM AND SAVE
!   THE INTERMEDIATE TRIDIAGONAL MATRIX IN PARAMETER d_e .
!
    call eigval_cmp( vec, d, failure, upper=upper, sort=sort, d_e=d_e )
!
!   COMPUTE THE FIRST m EIGENVECTORS OF THE SELF-ADJOINT MATRIX a BY
!   maxiter INVERSE ITERATIONS ON THE INTERMEDIATE TRIDIAGONAL MATRIX
!   d_e AND BACK-TRANSFORMATION.
!
    call trid_inviter( d_e(:n,1), d_e(:n,2), d(:m), eigvec, failure2,     &
                        matp=vec, maxiter=maxiter )
!
!   CHECK FOR SMALL RESIDUALS OF THE EXPRESSION a*eigvec - eigvec*d
!   WHERE d ARE THE EIGENVALUES AND eigvec THE EIGENVECTORS OF a.
!
    err = sum( abs(matmul(a,eigvec)-eigvec*spread(d(:m),1,n)) )/sum( abs(d(:m)) )
    if ( err<=sqrt(epsilon(err)) .and. .not.failure  .and. .not.failure2 ) then
        write (prtunit,*) 'Example 2 of EIGVAL_CMP is correct'
    else
        write (prtunit,*) 'Example 2 of EIGVAL_CMP is incorrect'
    end if
!
!
! END OF PROGRAM ex2_eigval_cmp
! =============================
!
end program ex2_eigval_cmp
