program ex2_probq
!
!
! Purpose
! =======
!
!   This program is intended to illustrate the use of functions PROBQ, PINVQ
!   in module Prob_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 26/02/2013
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, false, one, c99, probq, pinvq
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=5000, m=10000
!
    real(stnd), parameter :: eps = 1.0e-3_stnd 
!
    character(len=*), parameter :: name_proc='Example 2 of probq'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), dimension(n,m) :: p, p2, x2
    real(stnd)                 :: err
!
    integer(i4b), dimension(n,m) :: ndf
!
    logical(lgl) :: upper
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE THE DEGREES OF FREEDOM OF CHI-SQUARED DISTRIBUTION .
!
    call random_number( x2(:n,:m) )
!
    x2(:n,:m) = c99*x2(:n,:m)
    where( x2(:n,:m)<one )  x2(:n,:m) = x2(:n,:m) + one
!
    ndf(:n,:m) = x2(:n,:m)
!
!   GENERATE A RANDOM PROBABILITY MATRIX p(:m,:n) .
!
    call random_number( p(:n,:m) )
!
!   COMPUTE THE QUANTILES x2(:m,:n) OF CHI-SQUARED DISTRIBUTION WITH ndf DEGREES OF FREEDOM
!   CORRESPONDING TO LOWER TAIL AREAS OF p(:m,:n) .
!
    x2(:n,:m) = pinvq( p(:n,:m), ndf(:n,:m) )
!
!   RECOMPUTE THE PROBABILITIES FROM THE QUANTILES
!   WITH probq FUNCTION.
!
    upper = false
!
    p2(:n,:m) = probq( x2(:n,:m), ndf(:n,:m), upper=upper )
!
!   CHECK THAT p(:n,:m) AND p2(:n,:m) AGREE.
!
    err = maxval( abs( p(:n,:m) - p2(:n,:m) ) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=eps ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex2_probq
! ========================
!
end program ex2_probq
