program ex2_probt
!
!
! Purpose
! =======
!
!   This program is intended to illustrate the use of functions PROBT, PINVT
!   in module Prob_Procedures .
!                                                                              
!                                                                              
! LATEST REVISION : 26/02/2013
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : lgl, i4b, stnd, false, one, c99, probt, pinvt
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=5000, m=10000
!
    real(stnd), parameter :: eps = 1.0e-4_stnd 
!
    character(len=*), parameter :: name_proc='Example 2 of probt'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), dimension(n,m) :: p, p2, t
    real(stnd)                 :: err
!
    integer(i4b), dimension(n,m) :: ndf
!
    logical(lgl) :: upper
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE EXAMPLE.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   GENERATE THE DEGREES OF FREEDOM OF STUDENT T-DISTRIBUTION .
!
    call random_number( t(:n,:m) )
!
    t(:n,:m) = c99*t(:n,:m)
    where( t(:n,:m)<one )  t(:n,:m) = t(:n,:m) + one
!
    ndf(:n,:m) = t(:n,:m)
!
!   GENERATE A RANDOM PROBABILITY MATRIX p(:,:) .
!
    call random_number( p(:n,:m) )
!
!   COMPUTE THE QUANTILES t(:,:) OF STUDENT T-DISTRIBUTION WITH ndf(:,:) DEGREES OF FREEDOM
!   CORRESPONDING TO LOWER TAIL AREAS OF p(:,:) .
!
    t(:n,:m) = pinvt( p(:n,:m), ndf(:n,:m) )
!
!   RECOMPUTE THE PROBABILITIES FROM THE QUANTILES
!   WITH probt FUNCTION.
!
    upper = false
!
    p2(:n,:m) = probt( t(:n,:m), ndf(:n,:m), upper=upper )
!
!   CHECK THAT p AND p2 AGREE.
!
    err = maxval( abs( p(:n,:m) - p2(:n,:m) ) )
!
!   PRINT THE RESULT OF THE TEST.
!
    if ( err<=eps ) then
        write (prtunit,*) name_proc//' is correct'
    else
        write (prtunit,*) name_proc//' is incorrect'
    end if
!
!
! END OF PROGRAM ex2_probt
! ========================
!
end program ex2_probt
