####################################################################
#  STATPACK make include file for GNU GFORTRAN compiler            #
#                                                                  #
#  STATPACK, Version 2                                             #
#  September, 2018                                                 #
####################################################################
#
SHELL = /bin/sh
#
#
#  Give here the absolute path to the STATPACK library directory.
#
DIRLIB = /path/to/statpack/library/directory
#
#  Give here the name for the STATPACK library.
#
LIB = /statpack/library/name
#
#  If you use (for example) 
#
#                  LIB = _statpack
#
#  The static library name will be lib_statpack.a . The library is
#  created in the $(DIRLIB) directory.
#
#  The shared library name will be lib_statpack.so . The shared library is also
#  created in the $(DIRLIB) directory.
#
#  Give here the absolute path to the interfaces directory (i.e. directory for the .mod files
#  generated by the compiler), if needed.
#
INTERFACES = /path/to/statpack/interfaces/directory
#  
#  Modify the FORTRAN, OPTS, NOOPTFLAGS, FLAGS, DRVFLAGS and 
#  CHECKFLAGS definitions to refer to the compiler
#  and desired compiler options for your machine.
#
#  Give here the name of the fortran90 compiler for your machine.
#
FORTRAN    = gfortran
#
#  This variable is for compilers that need a prefix before the -D flag (i.e., xlf)
#
DEFINE_PREPEND = 
#
#  OPTS refers to the compiler options desired in all cases.
#  Remove the -fopenmp flag if you want a sequential version
#  Add the -mcmodel=medium flag if you need to process huge data.
#
OPTS       = -ffree-form -J$(INTERFACES) -fopenmp -fPIC \
             $(DEFINE_PREPEND)-D_DOT_PRODUCT            \
             $(DEFINE_PREPEND)-D_MATMUL                 \
             $(DEFINE_PREPEND)-D_TRANSPOSE              \
             $(DEFINE_PREPEND)-D_USE_GNU                \
             $(DEFINE_PREPEND)-D_F2003
#
#  NOOPTFLAGS refers to the compiler options desired when
#  NO OPTIMIZATION is selected for compilation of some modules in STATPACK.
#
NOOPTFLAGS = -O0
#
#  FLAGS refers to the compiler options desired when
#  OPTIMIZATION is selected for compilation of modules in STATPACK.
#
FLAGS = -O2
#
#  CHECKFLAGS refers to the compiler options desired when
#  a syntax check of the fortran in the source files is done.
#
CHECKFLAGS = -fsyntax-only -std=f2003
#
#  DRVFLAGS refers to the compiler options desired for running the tests 
#  and examples of the routines in the library.
#
DRVFLAGS = $(FLAGS)
#
#  The archiver and the flag(s) to use when building archive (library)
#  If you system has no ranlib, set RANLIB = echo.
#
ARCH     = ar
ARCHFLAGS= -ruv
RANLIB   = ranlib
#
#  Define here your BLAS library if you want to use one.
#  If not available, do not define this variable and remove
#  the -D_BLAS flag in the compiler flags (e.g. in the OPTS macro).
#
LBLAS = 
#
# For a static library, use something like:
#
#LBLAS = -L/path/to/blas/lib -lblas
#
# For a dynamic library, use something like:
#
#LBLAS = -Wl,-rpath,/path/to/blas/lib -L/path/to/blas/lib -lblas
#
#  Define here your LAPACK library if you want to use one.
#  If not available, do not define this variable.
#
LLAPACK = 
#
# For a static library, use something like:
#
#LLAPACK = -L/path/to/lapack/lib -llapack
#
# For a dynamic library, use something like:
#
#LLAPACK = -Wl,-rpath,/path/to/lalack/lib -L/path/to/lapack/lib -llapack
#
#  The linker and the flag(s) to use when building shared library.
#
LIBTOOL      = $(FORTRAN)
LIBTOOLFLAGS =  -shared -fopenmp  -fPIC  $(LLAPACK)  $(LBLAS)
#
#  LOADFLAGS refer to the desired load options for your machine. 
#
LOADFLAGS  =  -L$(DIRLIB) -l$(LIB)  $(LLAPACK)  $(LBLAS)
#
# For a dynamic library, use something like:
#
# LOADFLAGS  =  -Wl,-rpath,$(DIRLIB)  -L$(DIRLIB) -l$(LIB)  $(LLAPACK)  $(LBLAS)
