program test_kind
!! #############################################################################
!!
!! Copyright 2018 IRD
!!  
!! This file is part of statpack.
!!  
!! statpack is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!  
!! statpack is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!  
!! You can find a copy of the GNU Lesser General Public License
!! in the statpack/doc directory.
!!
!! #############################################################################
!
!
! Purpose
! =======
!
!   test_kind lists available kinds of logical, integer, real types
!   and properties of standard numerical types on the computer
!
!
! Further Details
! ===============
!
!   The program as given here must be modified before compiling if one of the logical 
!   kinds logic0, logic1, logic2, logic4 is not available. In this case, it is necessary
!   to comment out the write statements corresponding to the missing kind types, before
!   compiling test_kind.
!
!   LATEST REVISION : 10/09/2018
!
!
! ==========================================================================================
!
!
! USED MODULES 
! ============
!   
! SYMBOLIC NAMES FOR  KIND TYPES OF 'logical'.
!
    use The_Kinds, only : logic, logic0, logic1, logic2, logic4,         &
!
!
! SYMBOLIC NAMES FOR KIND TYPES OF 1-, 2-, 4- AND 8-BYTES INTEGERS.
!
                          i1b, i2b, i4b, i8b,                            &
!
!
! SYMBOLIC NAMES FOR KIND TYPES OF SINGLE- , DOUBLE- AND QUADRUPLE-PRECISION REALS
! AND SYMBOLIC NAMES FOR low, normal AND extended real IMPLEMENTATIONS.
!
                          sp, dp, qp, low, normal, extended
!
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!
!
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT.
!
    integer, parameter :: prtunit = 6
!   
    integer, parameter :: k_int = kind( 0 ), k_long = max( k_int,i8b ), k_quad = max( sp,qp )
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   FIRST, LIST AVAILABLE KINDS FOR INTRINSIC TYPES.
!
    write( prtunit, 10 ) logic
!
!   IF THE PROGRAM DO NOT COMPILE, ONE OR MORE OF THE FOLLOWING
!   logical KIND TYPES ARE NOT AVAILABLE. IN THIS CASE, YOU MUST
!   PUT A LEADING '!' ON THE CORRESPONDING STATEMENTS AND
!   RECOMPILE THE PROGRAM.
!
!    write( prtunit, 20 ) 'logic0', kind( logical( .true.,logic0 ) )
    write( prtunit, 20 ) 'logic1', kind( logical( .true.,logic1 ) )
    write( prtunit, 20 ) 'logic2', kind( logical( .true.,logic2 ) )
    write( prtunit, 20 ) 'logic4', kind( logical( .true.,logic4 ) )
!
!
    write( prtunit, 30 )                                               & 
!   
      k_int, 'i1b', i1b, 'i2b', i2b, 'i4b', i4b, 'i8b', i8b            &
!
    , sp, 'sp', sp, 'dp', dp, 'qp', qp                                 &
    , 'low', low, 'normal', normal, 'extended', extended
!
!   PRINT PROPERTIES OF NUMERICAL INTRINSIC TYPES.
!
    write( prtunit, 40 ) 'INTEGER', k_int
    write( prtunit, '(15x,4a30/)' ) '1-byte', '2-bytes', '4-bytes', '8-bytes'
    write( prtunit, 50 ) 'KIND', i1b, i2b, i4b, i8b
!
    if ( i8b==k_long ) then
!
        write( prtunit, 50 )                                                            &
!
    'HUGE' , huge(1_i1b),  huge(1_i2b),  huge(1_i4b),  huge(1_k_long),                  &
    'RANGE', range(1_i1b), range(1_i2b), range(1_i4b), range(1_k_long),                 &
    'BIT_SIZE', bit_size(1_i1b), bit_size(1_i2b), bit_size(1_i4b), bit_size(1_k_long),  &
    'RADIX', radix(1_i1b), radix(1_i2b), radix(1_i4b), radix(1_k_long),                 &
    'DIGITS', digits(1_i1b), digits(1_i2b), digits(1_i4b), digits(1_k_long)
!
    else
!
        write( prtunit, 60 )                                            &
!
    'HUGE' , huge(1_i1b),  huge(1_i2b),  huge(1_i4b),                   &
    'RANGE', range(1_i1b), range(1_i2b), range(1_i4b),                  &
    'BIT_SIZE', bit_size(1_i1b), bit_size(1_i2b), bit_size(1_i4b),      &
    'RADIX', radix(1_i1b), radix(1_i2b), radix(1_i4b),                  &
    'DIGITS', digits(1_i1b), digits(1_i2b), digits(1_i4b)
!
    end if
!
    write( prtunit, 40 ) 'REAL', sp
    write( prtunit, '(a15,3a30/)' ) 'Precision', 'simple', 'double', 'quadruple'
    write( prtunit, 60 ) 'KIND', sp, dp, qp
!
    if ( qp==k_quad ) then
!
        write( prtunit, 60)                                                             &
!
    'PRECISION' , precision(1._sp),  precision(1._dp), precision(1._k_quad),            &
    'RANGE', range(1._sp), range(1._dp), range(1._k_quad),                              &
    'EPSILON' , epsilon(1._sp),  epsilon(1._dp), epsilon(1._k_quad),                    &
    'HUGE' , huge(1._sp),  huge(1._dp), huge(1._k_quad),                                &
    'TINY' , tiny(1._sp),  tiny(1._dp), tiny(1._k_quad),                                & 
    'RADIX' , radix(1._sp),  radix(1._dp), radix(1._k_quad),                            &
    'DIGITS' , digits(1._sp),  digits(1._dp), digits(1._k_quad),                        &
    'MAXEXPONENT' , maxexponent(1._sp),  maxexponent(1._dp), maxexponent(1._k_quad),    &
    'MINEXPONENT' , minexponent(1._sp),  minexponent(1._dp), minexponent(1._k_quad)
!
    else
!
        write( prtunit, 70 )                                     &
!
    'PRECISION' , precision(1._sp),  precision(1._dp),           &
    'RANGE', range(1._sp), range(1._dp),                         &
    'EPSILON' , epsilon(1._sp),  epsilon(1._dp),                 &
    'HUGE' , huge(1._sp),  huge(1._dp),                          &
    'TINY' , tiny(1._sp),  tiny(1._dp),                          &
    'RADIX' , radix(1._sp),  radix(1._dp),                       &
    'DIGITS' , digits(1._sp),  digits(1._dp),                    &
    'MAXEXPONENT' , maxexponent(1._sp),  maxexponent(1._dp),     &
    'MINEXPONENT' , minexponent(1._sp),  minexponent(1._dp)
!
    end if
!
!
! FORMATS 
! =======
!
    10  format(// ' **********************************************'/   &
                  ' * List of symbolic names for avalaible kinds *'/   &
                  ' **********************************************'/// &
                  ' LOGICAL types'/                                    &
                  ' ============='//                                   &
                  ' Kind type of default logical = ',i3, /             &
                  ' ----------------------------------'  //            &
                  ' Symbolic names for kinds of logicals'/             &
                  ' ------------------------------------'/  )
!
    20  format(1x, a, ' = ', i3)
!
    30  format(// ' INTEGER types'/                                                  &
                  ' ============='//                                                 &
                  ' Kind type of default integer = ',i3, /                           &
                  ' ----------------------------------'  //                          &
                  ' Symbolic names for kinds of 1-, 2-, 4- and 8-bytes integers'/    &
                  ' -----------------------------------------------------------'/    &
                  , 4(a,' = ',i3,/),                                                 &
               // ' REAL types'/                                                     &
                  ' =========='//                                                    &
                  ' Kind type of default real = ',i3, /                              &
                  ' -------------------------------'  //                             &
                  ' Symbolic names for kinds of reals'/                              &
                  ' ---------------------------------'/                              &
                  , 6(a8,' = ',i3,/),                                                &
             ////  ' **********************************************'/                &
                  ,' *  Properties of standard numerical types    *'/                &
                  ,' **********************************************'   ) 
!
    40  format(//' type ',a,5x,'default KIND =',i3 /)
!
    50  format(1x,a11,' = ',4g30.8)
!
    60  format(1x,a11,' = ',3g30.16E4)
!
    70  format(1x,a11,' = ',2g30.16)
!
!
! END OF PROGRAM test_kind
! ========================
!
end program test_kind
