program test_comp_cormat_miss
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine comp_cormat_miss in module Mul_Stat_Procedures .
!
! LATEST REVISION : 20/04/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, true, false, zero, one, comp_cormat_miss, triangle
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=20, p=50
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                           :: err_mean, err_std, err_cor, err, eps, miss=-999.99_stnd
    real(stnd), dimension(m,m)           :: cor1, cov1, cor2
    real(stnd), dimension(m)             :: mean1, std1, std2, xn1
    real(stnd), dimension(m,2)           :: mean2
    real(stnd), dimension((m*(m+1))/2)   :: corp1, covp1, corp2
    real(stnd), dimension((m*(m+1))/2,3) :: xn2
    real(stnd), dimension(m,p)           :: x, xt, temp
!
    integer(i4b) :: i, k, k2, inc, j
!
    logical(lgl)                 :: first, last, cov
    logical(lgl), dimension(m,p) :: xmask, mask
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_cormat_miss'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x WITH MISSING VALUES.
!
    call random_number( x )
!
    xmask = x>0.05_stnd
    where ( .not.xmask ) x = miss
!
!   COMPUTE THE MEANS AND STANDARD-DEVIATIONS OF x  .
!
    xn1 = count( xmask, dim=2_i4b )
!
    mean1 = sum( x, dim=2_i4b, mask=xmask )/xn1
    where ( xmask ) 
        xt = x - spread( mean1, dim=2_i4b, ncopies=p )
    elsewhere
        xt = miss
    end where
    std1 = sqrt( sum( xt**2, mask=xmask, dim=2_i4b )/xn1 )
!
!   COMPUTE THE COVARIANCE MATRIX.
!
!BUG:   SOME COMPILERS (eg PGF90) DO NOT IMPLEMENT THIS SECTION OF CODE
!       CORRECTLY. THUS, IT IS SAFER TO USE ANOTHER SECTION OF CODE.
!
!    cov1 = zero
!    do k = 1_i4b, m
!        mask(k:m,:) = xmask(k:m,:) .and. spread(xmask(k,:),dim=1_i4b, ncopies=m-k+1_i4b )
!        cov1(k:m,k) = sum( xt(k:m,:)*spread(xt(k,:),dim=1_i4b, ncopies=m-k+1_i4b ),       &
!                           mask=mask(k:m,:), dim=2_i4b )
!        cov1(k:m,k) = cov1(k:m,k)/count( mask(k:m,:), dim=2_i4b )
!        cov1(k,k+1_i4b:m) = cov1(k+1_i4b:m,k)
!    end do
!
    cov1 = zero
    do k = 1_i4b, m
        j = m-k+1_i4b
        mask(1_i4b:j,1_i4b:p) = xmask(k:m,1_i4b:p) .and. spread(xmask(k,1_i4b:p),dim=1_i4b, ncopies=j )
        temp(1_i4b:j,1_i4b:p) = xt(k:m,1_i4b:p)*spread(xt(k,1_i4b:p),dim=1_i4b, ncopies=j )
        cov1(k:m,k) = sum( temp(1_i4b:j,1_i4b:p), mask=mask(1_i4b:j,1_i4b:p), dim=2_i4b )/   &
                      real( count( mask(1_i4b:j,1_i4b:p), dim=2_i4b ), stnd )
        cov1(k,k+1_i4b:m) = cov1(k+1_i4b:m,k)
    end do
    covp1 = pack( cov1, mask=triangle( true, m, m, extra=1_i4b) )
!
!   COMPUTE THE CORRELATION MATRIX  .
!
    std2 = one/std1
    cor1 = spread( std2, dim=2_i4b, ncopies=m )*cov1
    cor1 = cor1*spread( std2, dim=1_i4b, ncopies=m )
    corp1 = pack( cor1, mask=triangle( true, m, m, extra=1_i4b) )
!
    cov = false
!
    do i = 1_i4b, 8_i4b
!
        cov = .not.cov
        inc = merge( 1_i4b, p, i<=4_i4b )
!
        select case( i )
!
            case( 1_i4b, 2_i4b, 5_i4b, 6_i4b  )
!
!               TESTS 1, 2, 5 AND 6 :
!
!               COMPUTE THE COVARIANCE OR CORRELATION MATRIX WITH comp_cormat_miss .
!
                k2 = 0_i4b
                do k = 1_i4b, p, inc
!
                    k2 = min( k2+inc, p )
!
                    first = k==1_i4b
                    last = k2==p
!
                    call comp_cormat_miss( x(:,k:k2), first, last, mean2, cor2, xn2, miss,    &
                                           xstd=std2, cov=cov, fill=true )
                end do
!
                if ( cov ) then
                   err_cor  = maxval( abs( cov1-cor2 ) )
                else
                    err_cor  = maxval( abs( cor1-cor2 ) )
                end if
!
            case( 3_i4b, 4_i4b, 7_i4b, 8_i4b )
!
!               TESTS 3, 4, 7 AND 8 :
!
!               COMPUTE THE COVARIANCE OR CORRELATION MATRIX WITH comp_cormat_miss_b .
!
                k2 = 0_i4b
                do k = 1_i4b, p, inc
!
                    k2 = min( k2+inc, p )
!
                    first = k==1_i4b
                    last = k2==p
!
                    call comp_cormat_miss( x(:,k:k2), first, last, mean2, corp2, xn2, miss,    &
                                           xstd=std2, cov=cov )
                end do
!
                if ( cov ) then
                    err_cor  = maxval( abs( covp1-corp2 ) )
                else
                    err_cor  = maxval( abs( corp1-corp2 ) )
                end if
!
        end select
!
!       CHECK THAT THE TWO SETS OF STATISTICS AGREE.
!
        err_mean = maxval( abs( (mean1-mean2(:,1_i4b))/mean2(:,1_i4b) ) )
        err_std  = maxval( abs( (std1-std2)/std2 ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max( err_mean, err_std, err_cor )
!
        if ( err<=eps ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
!
    end do
!
! END OF PROGRAM test_comp_cormat_miss
! ====================================
!
end program test_comp_cormat_miss
