program test_comp_eof
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine comp_eof in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 05/07/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, false, one, comp_eof
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=20, p=50
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                         :: err_mean, err_std, err_eig, err, eps, xn
    real(stnd), dimension(m,m)         :: eigvec, cor1, cov1
    real(stnd), dimension(m)           :: mean1, mean2, std1, std2, eigval
    real(stnd), dimension(m,p)         :: x, xt
!
    integer(i4b) :: i, k, k2, inc
!
    logical(lgl) :: first, last, cov, failure
!
    character    :: sort = 'd'
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_eof'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x .
!
    call random_number( x )
!
!   COMPUTE THE MEANS AND STANDARD-DEVIATIONS OF x  .
!
    mean1 = sum( x, dim=2_i4b )/p
    xt = x - spread( mean1, dim=2_i4b, ncopies=p )
    std1 = sqrt( sum( xt**2, dim=2_i4b )/p )
!
!   COMPUTE THE COVARIANCE  MATRIX  .
!
    cov1 = matmul( xt, transpose( xt ) )/p
!
!   COMPUTE THE CORRELATION MATRIX .
!
    std2 = one/std1
    cor1 = cov1*spread( std2, dim=2_i4b, ncopies=m )
    cor1 = cor1*spread( std2, dim=1_i4b, ncopies=m )
!
    cov = false
!
    do i = 1_i4b, 4_i4b
!
        cov = .not.cov
        inc = merge( 1_i4b, p, i<=2_i4b )
!
!       COMPUTE EOFs FROM THE COVARIANCE OR CORRELATION MATRIX .
!
        k2 = 0_i4b
        do k = 1_i4b, p, inc
!
            k2 = min( k2+inc, p )
!
            first = k==1_i4b
            last = k2==p
!
            call comp_eof( x(:,k:k2), first, last, eigval, eigvec, xn, failure,   &
                           cov=cov, sort=sort, xmean=mean2, xstd=std2      )
        end do
!
        if ( cov ) then
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION cov1*eigvec - eigvec*eigvval
!           WHERE eigvval ARE THE EIGENVALUES AND eigvec THE EIGENVECTORS OF cov1.
!
            err_eig =         &
            sum( abs(matmul(cov1,eigvec)-eigvec*spread(eigval,1,m)) )/sum( eigval )
        else
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION cor1*eigvec - eigvec*eigvval
!           WHERE eigvval ARE THE EIGENVALUES AND eigvec THE EIGENVECTORS OF cor1.
!
            err_eig =         &
            sum( abs(matmul(cor1,eigvec)-eigvec*spread(eigval,1,m)) )/sum( eigval )
        end if
!
!       CHECK THAT THE TWO SETS OF UNIVARIATE STATISTICS AGREE.
!
        err_mean = maxval( abs( (mean1-mean2)/mean2 ) )
        err_std  = maxval( abs( (std1-std2)/std2 ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max( err_mean, err_std, err_eig )
!
        if ( err<=eps .and. .not.failure ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
    end do
!
!
! END OF PROGRAM test_comp_eof
! ============================
!
end program test_comp_eof
