program test_comp_eof3
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine COMP_EOF3 in module Mul_Stat_Procedures .
!                                                                              
! LATEST REVISION : 05/07/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, comp_eof3
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=50, neig=3
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                         :: err_eig, eps
    real(stnd), dimension(m,m)         :: sscp, x
    real(stnd), dimension(m,neig)      :: eigvec
    real(stnd), dimension(m,2)         :: eigval
!
    integer(i4b) :: i, maxiter=4
!
    logical(lgl) :: failure
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_eof3'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x .
!
    call random_number( x )
!
    do i = 1_i4b, 2_i4b
!
        call comp_eof3( x(:m,:m), i, failure, xeigval=eigval, xeigvec=eigvec, maxiter=maxiter )
!
!       COMPUTE THE SSCP MATRIX  .
!
        if ( i==1_i4b ) then
            sscp = matmul( x, transpose( x ) )/m
        else
            sscp = matmul( transpose( x ), x  )/m
        end if
!
!       CHECK FOR SMALL RESIDUALS OF THE EXPRESSION sscp*eigvec - eigvec*eigvval
!       WHERE eigvval ARE THE EIGENVALUES AND eigvec THE EIGENVECTORS OF sscp.
!
        err_eig =         &
        sum( abs(matmul(sscp,eigvec)-eigvec*spread(eigval(:neig,1),1,m)) )/sum( eigval(:neig,1) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        if ( err_eig<=eps .and. .not.failure ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err_eig,' > ',eps
        end if
!
    end do
!
!
! END OF PROGRAM test_comp_eof3
! =============================
!
end program test_comp_eof3
