program test_comp_eof_miss2
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine comp_eof_miss2 in module Mul_Stat_Procedures .
!
! LATEST REVISION : 05/07/2010
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, lgl, false, zero, one, comp_eof_miss2
#ifdef _MATMUL
    use Statpack, only : matmul=>matmul2
#endif
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, m=20, p=50, neig=3
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                           :: err_mean, err_std, err_eig, err, eps, miss=-999.99_stnd
    real(stnd), dimension(m,m)           :: cor1, cov1
    real(stnd), dimension(m,neig)        :: eigvec
    real(stnd), dimension(m)             :: mean1, mean2, std1, std2, xn
    real(stnd), dimension(m,2)           :: eigval
    real(stnd), dimension(m,p)           :: x, xt
    real(stnd), dimension((m*(m+1))/2,4) :: corp
!
    integer(i4b) :: i, j, k, k2, inc, maxiter=3
!
    logical(lgl)                 :: first, last, cov, failure
    logical(lgl), dimension(p)   :: mask
    logical(lgl), dimension(m,p) :: xmask
!   
!   
! PARAMETERS 
! ==========
!
    character(len=*), parameter :: name_proc='comp_eof_miss2'
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    eps = sqrt( epsilon(eps) )
!
!   GENERATE A RANDOM OBSERVATION ARRAY x WITH MISSING VALUES.
!
    call random_number( x )
!
    xmask = x>0.05_stnd
    where ( .not.xmask ) x = miss
!
!   COMPUTE THE MEANS AND STANDARD-DEVIATIONS OF x  .
!
    xn = count( xmask, dim=2_i4b )
!
    mean1 = sum( x, dim=2_i4b, mask=xmask )/xn
    where ( xmask ) 
        xt = x - spread( mean1, dim=2_i4b, ncopies=p )
    elsewhere
        xt = miss
    end where
    std1 = sqrt( sum( xt**2, mask=xmask, dim=2_i4b )/xn )
!
!   COMPUTE THE COVARIANCE MATRIX.
!
!BUG:   SOME COMPILERS (eg PGF90) DO NOT IMPLEMENT THIS SECTION OF CODE
!       CORRECTLY. THUS, IT IS SAFER TO USE ANOTHER SECTION OF CODE.
!
!    cov1 = zero
!    do k = 1_i4b, m
!        mask(k:m,:) = xmask(k:m,:) .and. spread(xmask(k,:),dim=1_i4b, ncopies=m-k+1_i4b )
!        cov1(k:m,k) = sum( xt(k:m,:)*spread(xt(k,:),dim=1_i4b, ncopies=m-k+1_i4b ),       &
!                           mask=mask(k:m,:), dim=2_i4b )
!        cov1(k:m,k) = cov1(k:m,k)/count( mask(k:m,:), dim=2_i4b )
!        cov1(k,k+1_i4b:m) = cov1(k+1_i4b:m,k)
!    end do
!
    cov1(:m,:m) = zero
!
    do k = 1_i4b, m
!
        do j = k, m
            mask(:p)  = xmask(j,:p) .and. xmask(k,:p)
            cov1(j,k) = sum( xt(j,:p)*xt(k,:p), mask=mask(:p) )
            cov1(j,k) = cov1(j,k)/count( mask(:p) )
            
        end do
!
        cov1(k,k+1_i4b:m) = cov1(k+1_i4b:m,k)
!
    end do
!
!   COMPUTE THE CORRELATION MATRIX  .
!
    std2 = one/std1
    cor1(:m,:m) = spread( std2, dim=2_i4b, ncopies=m )*cov1(:m,:m)
    cor1(:m,:m) = cor1(:m,:m)*spread( std2, dim=1_i4b, ncopies=m )
!
    cov = false
!
    do i = 1_i4b, 4_i4b
!
        cov = .not.cov
        inc = merge( 1_i4b, p, i<=2_i4b )
!
!       COMPUTE EOFs FROM THE COVARIANCE OR CORRELATION MATRIX .
!
        k2 = 0_i4b
        do k = 1_i4b, p, inc
!
            k2 = min( k2+inc, p )
!
            first = k==1_i4b
            last = k2==p
!
            call comp_eof_miss2( x(:,k:k2), first, last, eigval,  corp, miss, failure,   &
                                cov=cov, maxiter=maxiter, xmean=mean2, xstd=std2, xeigvec=eigvec )
        end do
!
        if ( cov ) then
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION cov1*eigvec - eigvec*eigvval
!           WHERE eigvval ARE THE EIGENVALUES AND eigvec THE EIGENVECTORS OF cov1.
!
            err_eig =         &
            sum( abs(matmul(cov1,eigvec)-eigvec*spread(eigval(:neig,1),1,m)) )/sum( eigval(:neig,1) )
        else
!
!           CHECK FOR SMALL RESIDUALS OF THE EXPRESSION cor1*eigvec - eigvec*eigvval
!           WHERE eigvval ARE THE EIGENVALUES AND eigvec THE EIGENVECTORS OF cor1.
!
            err_eig =         &
            sum( abs(matmul(cor1,eigvec)-eigvec*spread(eigval(:neig,1),1,m)) )/sum( eigval(:neig,1) )
        end if
!
!       CHECK THAT THE TWO SETS OF UNIVARIATE STATISTICS AGREE.
!
        err_mean = maxval( abs( (mean1-mean2)/mean2 ) )
        err_std  = maxval( abs( (std1-std2)/std2 ) )
!
!       PRINT THE RESULTS OF THE TESTS.
!
        err = max( err_mean, err_std, err_eig )
!
        if ( err<=eps .and. .not.failure ) then
            write (prtunit,*) 'Test ',i,' is correct'
        else
            write (prtunit,*) 'Test ',i,' is incorrect'
            write (prtunit,*) '=> ',err,' > ',eps
        end if
!
    end do
!
!
! END OF PROGRAM test_comp_eof_miss2
! ==================================
!
end program test_comp_eof_miss2
