program test_fft
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine FFT in module FFT_Procedures.
!                                                                              
!                                                                              
! LATEST REVISION : 26/09/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, n1_def, n2_def, n3_def,                   &
                         true, false, zero, one, half, c2000, init_fft, fft,  &
                         end_fft, lamch, norm, lassq
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=n1_def, n2=n2_def, n3=n3_def
!   
    real(stnd), parameter  :: fudge=c2000
!
    character(len=*), parameter :: name_proc='fft'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                                :: err, eps, ulp, safmin, tmp, ssq, scal, &
                                                 ssq_res, scal_res
    real(stnd), allocatable, dimension(:,:,:) :: x, y
!
    complex(stnd), allocatable, dimension(:)     :: tmpvec, tmpvec2
    complex(stnd), allocatable, dimension(:,:,:) :: a, c
!   
    integer(i4b), dimension(3) :: init_vec
    integer(i4b)               :: i, j, k, dim, dim2, m, n, success
    integer                    :: iok
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    ulp    = epsilon( err )
    eps    = fudge*ulp
    safmin = lamch( 'S' )
!
    success = 0_i4b
    m       = 0_i4b
!
    do n = n1, n2, n3
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( x(n,n,n), y(n,n,n), a(n,n,n), stat=iok )
!
        if ( iok/=0 ) cycle
!
!       GENERATE A COMPLEX RANDOM NUMBER ARRAY.
!
        call random_number( x(:n,:n,:n) )
!
        x(:n,:n,:n) = x(:n,:n,:n) - half
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            x(:n,:n,:n) = x(:n,:n,:n)/tmp
        end if
!
        call random_number( y(:n,:n,:n) )
!
        y(:n,:n,:n) = y(:n,:n,:n) - half
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            y(:n,:n,:n) = y(:n,:n,:n)/tmp
        end if
!
        a(:n,:n,:n) = cmplx( x(:n,:n,:n), y(:n,:n,:n), kind=stnd )
!
        deallocate( x, y )
!
!       ALLOCATE COMPLEX ARRAYS.
!
        allocate( c(n,n,n), tmpvec(n), tmpvec2(n), stat=iok )
!
        if ( iok/=0 ) cycle
!
!       SAVE THE COMPLEX RANDOM NUMBER ARRAY.
!
        c(:n,:n,:n) = a(:n,:n,:n)
!
        init_vec(:3_i4b) = (/ n, n, n /)
!
        do j = 1_i4b, 8_i4b
!
            m = m + 1_i4b
!
            select case( j )
!
            case( 1_i4b)
!
!               TEST 1 : TEST 1D FFT.
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                call init_fft( n )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fft( a(:n,1_i4b,1_i4b), forward=true  )
                call fft( a(:n,1_i4b,1_i4b), forward=false )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                a(:n,1_i4b,1_i4b) = a(:n,1_i4b,1_i4b) - c(:n,1_i4b,1_i4b)
!
                err = norm( a(:n,1_i4b,1_i4b) )/norm( c(:n,1_i4b,1_i4b) )
!
                a(:n,1_i4b,1_i4b) = c(:n,1_i4b,1_i4b)
!
            case( 2_i4b)
!
!               TEST 2 : TEST 2D FFT.
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                call init_fft( n, n )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fft( a(:n,:n,1_i4b), forward=true  )
                call fft( a(:n,:n,1_i4b), forward=false )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                a(:n,:n,1_i4b) = a(:n,:n,1_i4b) - c(:n,:n,1_i4b)
!
                err = norm( a(:n,:n,1_i4b) )/norm( c(:n,:n,1_i4b) )
!
                a(:n,:n,1_i4b) = c(:n,:n,1_i4b)
!
            case( 3_i4b)
!
!               TEST 3 : TEST 3D FFT.
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                call init_fft( n, n, n )
!    
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fft( a(:n,:n,:n), forward=true  )
                call fft( a(:n,:n,:n), forward=false )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                scal = one
                ssq  = zero
!
                scal_res = one
                ssq_res  = zero
!
                do i = 1_i4b, n
!
                    a(:n,:n,i) = a(:n,:n,i) - c(:n,:n,i)
!
                    call lassq( a(:n,:n,i), scal_res, ssq_res )
!
                    call lassq( c(:n,:n,i), scal, ssq )
!
                    a(:n,:n,i) = c(:n,:n,i)
!
                end do
!
                err = sqrt( ssq_res/ssq )*( scal_res/scal )
!
            case( 4_i4b, 5_i4b )
!
!               TESTS 4 AND 5 : TEST FFT_DIM_CM SUBROUTINE.
!
                if ( j==4_i4b ) then
                    dim  = 1_i4b
                    dim2 = 2_i4b
                else
                    dim  = 2_i4b
                    dim2 = 1_i4b
                end if
!
!               INITIALIZE THE FFT SUBROUTINE.
!    
                call init_fft( init_vec(:2_i4b), dim )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fft( a(:n,:n,1_i4b), forward=true  , dim=dim)
                call fft( a(:n,:n,1_i4b), forward=false , dim=dim)
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                a(:n,:n,1_i4b) = a(:n,:n,1_i4b) - c(:n,:n,1_i4b)
!
                err = maxval( norm( a(:n,:n,1_i4b), dim=dim2  )/norm( c(:n,:n,1_i4b), dim=dim2 ) )
!
                a(:n,:n,1_i4b) = c(:n,:n,1_i4b)
!
            case( 6_i4b, 7_i4b, 8_i4b )
!
!               TESTS 6, 7 AND 8 : TEST FFT_DIM_CT SUBROUTINE.
!
                if ( j==6_i4b ) then
                    dim = 1_i4b
                else if ( j==7_i4b ) then
                    dim = 2_i4b
                else
                    dim = 3_i4b
                end if
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                call init_fft( init_vec(:3_i4b), dim )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fft( a(:n,:n,:n), forward=true  , dim=dim)
                call fft( a(:n,:n,:n), forward=false , dim=dim)
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                err = zero
!
                do i = 1_i4b, n
!
                    do k = 1_i4b, n
!
                        select case( dim )
!
                        case( 1_i4b)
!
                            tmpvec(:n)  = c(:n,k,i)
                            tmpvec2(:n) = a(:n,k,i) - tmpvec(:n)
                            a(:n,k,i)   = tmpvec(:n)
!
                        case( 2_i4b)
!
                            tmpvec(:n)  = c(k,:n,i)
                            tmpvec2(:n) = a(k,:n,i) - tmpvec(:n)
                            a(k,:n,i)   = tmpvec(:n)
!
                        case( 3_i4b)
!
                            tmpvec(:n)  = c(k,i,:n)
                            tmpvec2(:n) = a(k,i,:n) - tmpvec(:n)
                            a(k,i,:n)   = tmpvec(:n)
!
                        end select
!
                        tmp = norm( tmpvec2(:n) )/norm( tmpvec(:n) )
!
                        err = max( err, tmp )                    
!
                    end do
!
                end do
!
            end select
!
!           PRINT THE RESULTS OF THE TESTS.
!
            if ( err<=eps ) then
!
                success = success + 1_i4b
!
            else
!
                write (prtunit,*) 'Test ',m,' is incorrect'
!
                write (prtunit,*) '=> ',err,' > ',eps,' , n= ', n,' , case= ', j
!
            end if
!
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        call end_fft()
!
        deallocate( a, c, tmpvec, tmpvec2 )
!
    end do
!
    if ( m/=success ) then
        write (prtunit,*) m - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test_fft
! =======================
!
end program test_fft
