program test_fftxy
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine FFTXY in module FFT_Procedures.
!                                                                              
!                                                                              
! LATEST REVISION : 21/05/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, n1_def, n2_def, n3_def,             &
                         false, zero, one, half, c1000, init_fft, fft,  &
                         fftxy, end_fft, lamch, norm, lassq
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n1=n1_def, n2=n2_def, n3=n3_def
!   
    real(stnd), parameter  :: fudge=c1000
!
    character(len=*), parameter :: name_proc='fftxy'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd)                                :: errx, erry, err, eps, eps2, ulp, safmin, tmp,  &
                                                 ssq, scal, ssq_res, scal_res
    real(stnd), allocatable, dimension(:)     :: tmpvec, tmpvec2
    real(stnd), allocatable, dimension(:,:,:) :: x, y, x2, y2
!
    complex(stnd), allocatable, dimension(:,:,:) :: fftx, ffty
!   
    integer(i4b), dimension(3) :: init_vec
    integer(i4b)               :: i, j, k, dim, dim2, m, n, success
    integer                    :: iok
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TESTS.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   SET THE REQUIRED PRECISION OF THE RESULTS.
!
    ulp    = epsilon( err )
    eps    = fudge*ulp
    safmin = lamch( 'S' )
!
    success = 0_i4b
    m       = 0_i4b
!
    do n = n1, n2, n3
!
!       ALLOCATE WORK ARRAYS.
!
        allocate( x(n,n,n),    y(n,n,n),      &
                  x2(n,n,n),   y2(n,n,n),     &
                  fftx(n,n,n), ffty(n,n,n),   &
                  tmpvec(n), tmpvec2(n),      &
                  stat=iok )
!
        if ( iok/=0 ) cycle
!
!       GENERATE TWO REAL RANDOM NUMBER ARRAYS.
!
        call random_number( x(:n,:n,:n) )
!
        x(:n,:n,:n) = x(:n,:n,:n) - half
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            x(:n,:n,:n) = x(:n,:n,:n)/tmp
        end if
!
        call random_number( y(:n,:n,:n) )
!
        y(:n,:n,:n) = y(:n,:n,:n) - half
!
        call random_number( tmp )
!
        if ( tmp>safmin ) then
            y(:n,:n,:n) = y(:n,:n,:n)/tmp
        end if
!
!       SAVE THE ARRAYS.
!
        x2(:n,:n,:n) = x(:n,:n,:n)
        y2(:n,:n,:n) = y(:n,:n,:n)
!
        init_vec(:3_i4b) = (/ n, n, n /)
!
        do j = 1_i4b, 8_i4b
!
            m = m + 1_i4b
!
            select case( j )
!
            case( 1_i4b)
!
!               TEST 1 : TEST 1D FFTXY.
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                call init_fft( n )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fftxy( x(:n,1_i4b,1_i4b), y(:n,1_i4b,1_i4b),            &
                            fftx(:n,1_i4b,1_i4b), ffty(:n,1_i4b,1_i4b)  )
!
                call fft( fftx(:n,1_i4b,1_i4b), forward=false )
                x(:n,1_i4b,1_i4b) = real( fftx(:n,1_i4b,1_i4b) )
!
                call fft( ffty(:n,1_i4b,1_i4b), forward=false )
                y(:n,1_i4b,1_i4b) = real( ffty(:n,1_i4b,1_i4b) )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                x(:n,1_i4b,1_i4b) = x(:n,1_i4b,1_i4b) - x2(:n,1_i4b,1_i4b)
!
                errx = norm( x(:n,1_i4b,1_i4b) )/norm( x2(:n,1_i4b,1_i4b) )
!
                x(:n,1_i4b,1_i4b) = x2(:n,1_i4b,1_i4b)
!
                y(:n,1_i4b,1_i4b) = y(:n,1_i4b,1_i4b) - y2(:n,1_i4b,1_i4b)
!
                erry = norm( y(:n,1_i4b,1_i4b) )/norm( y2(:n,1_i4b,1_i4b) )
!
                y(:n,1_i4b,1_i4b) = y2(:n,1_i4b,1_i4b)
!
            case( 2_i4b)
!
!               TEST 2 : TEST 2D FFTXY.
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                call init_fft( n, n )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fftxy( x(:n,:n,1_i4b), y(:n,:n,1_i4b),       &
                            fftx(:n,:n,1_i4b), ffty(:n,:n,1_i4b)  )
!
                call fft( fftx(:n,:n,1_i4b), forward=false )
                x(:n,:n,1_i4b) = real( fftx(:n,:n,1_i4b) )
!
                call fft( ffty(:n,:n,1_i4b), forward=false )
                y(:n,:n,1_i4b) = real( ffty(:n,:n,1_i4b) )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                x(:n,:n,1_i4b) = x(:n,:n,1_i4b) - x2(:n,:n,1_i4b)
!
                errx = norm( x(:n,:n,1_i4b) )/norm( x2(:n,:n,1_i4b) )
!
                x(:n,:n,1_i4b) = x2(:n,:n,1_i4b)
!
                y(:n,:n,1_i4b) = y(:n,:n,1_i4b) - y2(:n,:n,1_i4b)
!
                erry = norm( y(:n,:n,1_i4b) )/norm( y2(:n,:n,1_i4b) )
!
                y(:n,:n,1_i4b) = y2(:n,:n,1_i4b)
!
            case( 3_i4b)
!
!               TEST 3 : TEST 3D FFTXY.
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                call init_fft( n, n, n )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fftxy( x(:n,:n,:n), y(:n,:n,:n), fftx(:n,:n,:n), ffty(:n,:n,:n)  )
!
                call fft( fftx(:n,:n,:n), forward=false )
                x(:n,:n,:n) = real( fftx(:n,:n,:n) )
!
                call fft( ffty(:n,:n,:n), forward=false )
                y(:n,:n,:n) = real( ffty(:n,:n,:n) )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                scal = one
                ssq  = zero
!
                scal_res = one
                ssq_res  = zero
!
                do i = 1_i4b, n
!
                    x(:n,:n,i) = x(:n,:n,i) - x2(:n,:n,i)
!
                    call lassq( x(:n,:n,i), scal_res, ssq_res )
!
                    call lassq( x2 (:n,:n,i), scal, ssq )
!
                    x(:n,:n,i) = x2(:n,:n,i)
!
                end do
!
                errx = sqrt( ssq_res/ssq )*( scal_res/scal )
!
                scal = one
                ssq  = zero
!
                scal_res = one
                ssq_res  = zero
!
                do i = 1_i4b, n
!
                    y(:n,:n,i) = y(:n,:n,i) - y2(:n,:n,i)
!
                    call lassq( y(:n,:n,i), scal_res, ssq_res )
!
                    call lassq( y2(:n,:n,i), scal, ssq )
!
                    y(:n,:n,i) = y2(:n,:n,i)
!
                end do
!
                errx = sqrt( ssq_res/ssq )*( scal_res/scal )
!
            case( 4_i4b, 5_i4b )
!
!               TESTS 4 AND 5 : TEST FFTXY_DIM_CM SUBROUTINE.
!
                if ( j==4_i4b ) then
                    dim  = 1_i4b
                    dim2 = 2_i4b
                else
                    dim  = 2_i4b
                    dim2 = 1_i4b
                end if
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                CALL init_fft( init_vec(:2_i4b), dim )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fftxy( x(:n,:n,1_i4b), y(:n,:n,1_i4b),                &
                            fftx(:n,:n,1_i4b), ffty(:n,:n,1_i4b), dim=dim  )
!
                call fft( fftx(:n,:n,1_i4b), forward=false, dim=dim )
                x(:n,:n,1_i4b) = real( fftx(:n,:n,1_i4b) )
!
                call fft( ffty(:n,:n,1_i4b), forward=false, dim=dim )
                y(:n,:n,1_i4b) = real( ffty(:n,:n,1_i4b) )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                x(:n,:n,1_i4b) = x(:n,:n,1_i4b) - x2(:n,:n,1_i4b)
!
                errx = maxval( norm( x(:n,:n,1_i4b), dim=dim2  )/norm( x2(:n,:n,1_i4b), dim=dim2 ) )
!
                x(:n,:n,1_i4b) = x2(:n,:n,1_i4b)
!
                y(:n,:n,1_i4b) = y(:n,:n,1_i4b) - y2(:n,:n,1_i4b)
!
                erry = maxval( norm( y(:n,:n,1_i4b), dim=dim2  )/norm( y2(:n,:n,1_i4b), dim=dim2 ) )
!
                y(:n,:n,1_i4b) = y2(:n,:n,1_i4b)
!
            case( 6_i4b, 7_i4b, 8_i4b )
!
!               TESTS 6, 7 AND 8 : TEST FFTXY_DIM_CT SUBROUTINE.
!
                if ( j==6_i4b ) then
                    dim = 1_i4b
                else if ( j==7_i4b ) then
                    dim = 2_i4b
                else
                    dim = 3_i4b
                end if
!
!               INITIALIZE THE FFT SUBROUTINE.
!
                CALL init_fft( init_vec(:3_i4b), dim )
!
!               TRANSFORM AND THEN INVERT THE SEQUENCE BACK.
!
                call fftxy( x(:n,:n,:n), y(:n,:n,:n), fftx(:n,:n,:n), ffty(:n,:n,:n), dim=dim  )
!
                call fft( fftx(:n,:n,:n), forward=false, dim=dim )
                x(:n,:n,:n) = real( fftx(:n,:n,:n) )
!
                call fft( ffty(:n,:n,:n), forward=false, dim=dim )
                y(:n,:n,:n) = real( ffty(:n,:n,:n) )
!
!               CHECK THAT INVERSE(TRANSFORM(SEQUENCE)) = SEQUENCE.
!
                errx = zero
!
                do i = 1_i4b, n
!
                    do k = 1_i4b, n
!
                        select case( dim )
!
                        case( 1_i4b)
!
                            tmpvec(:n)  = x2(:n,k,i)
                            tmpvec2(:n) = x(:n,k,i) - tmpvec(:n)
                            x(:n,k,i)   = tmpvec(:n)
!
                        case( 2_i4b)
!
                            tmpvec(:n)  = x2(k,:n,i)
                            tmpvec2(:n) = x(k,:n,i) - tmpvec(:n)
                            x(k,:n,i)   = tmpvec(:n)
!
                        case( 3_i4b)
!
                            tmpvec(:n)  = x2(k,i,:n)
                            tmpvec2(:n) = x(k,i,:n) - tmpvec(:n)
                            x(k,i,:n)   = tmpvec(:n)
!
                        end select
!
                        tmp = norm( tmpvec2(:n) )/norm( tmpvec(:n) )
!
                        errx = max( errx, tmp )                    
!
                    end do
!
                end do
!
                erry = zero
!
                do i = 1_i4b, n
!
                    do k = 1_i4b, n
!
                        select case( dim )
!
                        case( 1_i4b)
!
                            tmpvec(:n)  = y2(:n,k,i)
                            tmpvec2(:n) = y(:n,k,i) - tmpvec(:n)
                            y(:n,k,i)   = tmpvec(:n)
!
                        case( 2_i4b)
!
                            tmpvec(:n)  = y2(k,:n,i)
                            tmpvec2(:n) = y(k,:n,i) - tmpvec(:n)
                            y(k,:n,i)   = tmpvec(:n)
!
                        case( 3_i4b)
!
                            tmpvec(:n)  = y2(k,i,:n)
                            tmpvec2(:n) = y(k,i,:n) - tmpvec(:n)
                            y(k,i,:n)   = tmpvec(:n)
!
                        end select
!
                        tmp = norm( tmpvec2(:n) )/norm( tmpvec(:n) )
!
                        erry = max( erry, tmp )                    
!
                    end do
!
                end do
!
            end select
!
            err = max( errx, erry )
!
            eps2 = real( n, stnd)*eps
!
!           PRINT THE RESULTS OF THE TESTS.
!
            if ( err<=eps2 ) then
!
                success = success + 1_i4b
!
            else
!
                write (prtunit,*) 'Test ',m,' is incorrect'
!
                write (prtunit,*) '=> ',err,' > ',eps2,' , n= ', n,' , case= ', j
!
            end if
!
        end do
!
!       DEALLOCATE WORK ARRAYS.
!
        call end_fft()
!
        deallocate( x, y, x2, y2, fftx, ffty, tmpvec, tmpvec2 )
!
    end do
!
    if ( m/=success ) then
        write (prtunit,*) m - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test_fftxy
! =========================
!
end program test_fftxy
