program test_quick_sort
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine QUICK_SORT in module Sort_Procedures.
!                                                                              
! LATEST REVISION : 16/05/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, false, arth, quick_sort, merror, allocate_error, n1_def
!   
!
! STRONG TYPING IMPOSED
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=max(100_i4b,n1_def)
!
    character(len=*), parameter :: name_proc='quick_sort'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), allocatable, dimension(:) :: x, x2
!
    integer                                 :: iok
    integer(i4b)                            :: i, j, k, success
    integer(i4b), allocatable, dimension(:) :: y, y2
!   
!
! EXECUTABLE STATEMENTS
! =====================
!
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( x(n), x2(n), y(n) ,y2(n), stat=iok  )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   GENERATE RANDOM REAL DATA TO SORT.
!
    call random_number( x2 )
!
!   GENERATE A RANDOM ORDERING OF THE INTEGERS 1 TO n.
!
    y2 = arth( 1_i4b, 1_i4b, n ) 
!    
!   STARTING AT THE END, SWAP THE CURRENT LAST INDICATOR WITH ONE
!   RANDOMLY CHOSEN FROM THOSE PRECEEDING IT.
!
    do i = n, 2_i4b, -1_i4b
!
        j = 1_i4b + i * x2(i)
!
        if (j < i) then
!
            k = y2(i)
            y2(i) = y2(j)
            y2(j) = k
!
        end if
!
    end do
!
!   SORT THE REAL AND INTEGER ARRAYS.
!
    success = 0_i4b
!
    do j = 1_i4b, 4_i4b
!
        if ( j<=2_i4b ) then
           x = x2
        else
           y = y2
        end if
!
        select case( j )
!
            case( 1_i4b )
!
!               TEST 1 : SORT THE REAL DATA IN ASCENDING ORDER.
!
                call quick_sort( x  )
!
!               CHECK THAT THE SORTED ARRAY IS NON-DECREASING.
!
                i = count( x(1:n-1) > x(2:n) )
!
            case( 2_i4b )
!
!               TEST 2 : SORT THE REAL DATA IN DECREASING ORDER.
!
                call quick_sort( x , ascending=false )
!
!               CHECK THAT THE SORTED ARRAY IS NON-INCREASING.
!
                i = count( x(1:n-1) < x(2:n) )
!
            case( 3_i4b )
!
!               TEST 3 : SORT THE INTEGER DATA IN ASCENDING ORDER.
!
                call quick_sort( y )
!
!               CHECK THAT THE SORTED ARRAY IS NON-DECREASING.
!
                i = count( y(1:n-1) > y(2:n) )
!
            case( 4_i4b )
!
!               TEST 4 : SORT THE INTEGER DATA IN DECREASING ORDER.
!
                call quick_sort( y, ascending=false )
!
!               CHECK THAT THE SORTED ARRAY IS NON-INCREASING.
!
                i = count( y(1:n-1) < y(2:n) )
!
        end select
!
!       PRINT THE RESULT OF THE TESTS.
!
        if ( i==0_i4b ) then
!
            success = success + 1_i4b
!
!            write (prtunit,*) 'Test ',j,' is correct'
!
        else
!
            write (prtunit,*) 'Test ',j,' is incorrect'
            write (prtunit,*) '=> ',i,' > ',0_i4b
!
        end if
!
    end do
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( x, x2, y, y2 )
!
    if ( success/=4_i4b ) then
        write (prtunit,*) 4_i4b - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test_quick_sort
! ==============================
!
end program test_quick_sort
