program test_replace_nan
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine and function NAN and REPLACE_NAN 
!   in module Num_Constants. 
!                                                                              
! LATEST REVISION : 10/04/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only :  i4b, lgl, stnd, replace_nan, nan, merror, allocate_error, n1_def, n2_def
!   
!
! STRONG TYPING IMPOSED 
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT, missing IS THE MISSING INDICATOR.
!
    integer(i4b), parameter :: prtunit=6, n=max(n1_def,10_i4b), p=max(n2_def,5_i4b)
!
    real(stnd), parameter :: missing=99.99_stnd
!
    character(len=*), parameter :: name_proc='replace_nan'
!   
!   
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), allocatable, dimension(:,:) :: mat
!
    integer      :: iok
    integer(i4b) :: j, success
!
    logical(lgl)   :: test
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   ALLOCATE WORK ARRAY.
!
    allocate( mat(n,p), stat=iok  )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   GENERATE A RANDOM DATA MATRIX.
!
    call random_number( mat ) 
!
!   TEST THE ROUTINES  replace_nan AND nan.
!
    success = 0_i4b
!
    do j = 1_i4b, 4_i4b
!
       mat(10_i4b,2_i4b) = nan()
!
        select case( j )
!
            case( 1_i4b )
!
!               TEST 1 : SCALAR.
!
                call replace_nan( mat(10_i4b,2_i4b), missing )
                test = any( mat==missing )
!
            case( 2_i4b )
!
!               TEST 2 : VECTOR.
!
                call replace_nan( mat(:n,2_i4b), missing )
                test = any( mat==missing )
!
            case( 3_i4b )
!
!               TEST 3 : MATRIX.
!
                call replace_nan( mat, missing )
                test = any( mat==missing )
!
            case( 4_i4b )
!
!               TEST 4 : REAL VECTOR WITHOUT A NAN.
!
                call replace_nan( mat(:n,1_i4b), missing )
                test = any( mat(:n,1_i4b)/=missing )
!
        end select
!
!       PRINT THE RESULTS OF THE TESTS.
!
        if ( test ) then
!
            success = success + 1_i4b
!
!            write (prtunit,*) 'Test ',j,' is correct'
!
        else
!
            write (prtunit,*) 'Test ',j,' is incorrect'
!
            write (prtunit,*)
!
        end if
!
    end do
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( mat )
!
    if ( success/=4_i4b ) then
        write (prtunit,*) 4_i4b - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test_replace_nan
! ===============================
!
end program test_replace_nan
