program test_tri_insert
!
!
! Purpose
! =======
!
!   This program is intended to test subroutine TRI_INSERT in module Sort_Procedures.
!                                                                              
! LATEST REVISION : 16/05/2018
!                                                                              
! ================================================================================================
!
!
! USED MODULES 
! ============
!
    use Statpack, only : i4b, stnd, arth, tri_insert, merror, allocate_error, n1_def
!   
!
! STRONG TYPING IMPOSED
! =====================
!    
    implicit none
!   
!   
! PARAMETERS 
! ==========
!
! prtunit IS THE PRINTING UNIT
!
    integer(i4b), parameter :: prtunit=6, n=max(100_i4b,n1_def)
!
    character(len=*), parameter :: name_proc='tri_insert'
!   
!
! SPECIFICATIONS FOR VARIABLES
! ============================
!
    real(stnd), allocatable, dimension(:) :: x
!
    integer                                 :: iok
    integer(i4b)                            :: i, j, k, success
    integer(i4b), allocatable, dimension(:) :: y
!   
!   
! EXECUTABLE STATEMENTS
! =====================
!
!   PRINT LABEL OF THE TEST.
!
    write (prtunit,*)
    write (prtunit,*) name_proc,' :'
    write (prtunit,*) repeat('*', len(name_proc) )
    write (prtunit,*)
!
!   ALLOCATE WORK ARRAYS.
!
    allocate( x(n), y(n), stat=iok  )
!
    if ( iok/=0 ) then
        call merror( name_proc//allocate_error )
    end if
!
!   GENERATE RANDOM REAL DATA TO SORT.
!
    call random_number( x )
!
!   GENERATE A RANDOM ORDERING OF THE INTEGERS 1 TO n.
!
    y = arth( 1_i4b, 1_i4b, n ) 
!    
!   STARTING AT THE END, SWAP THE CURRENT LAST INDICATOR WITH ONE
!   RANDOMLY CHOSEN FROM THOSE PRECEEDING IT.
!
    do i = n, 2_i4b, -1_i4b
!    
        j = 1_i4b + i * x(i)
!    
        if (j < i) then
!    
            k = y(i)
            y(i) = y(j)
            y(j) = k
!    
        end if
!    
    end do
!
!   SORT THE REAL AND INTEGER ARRAYS.
!
    success = 0_i4b
!
    do j = 1_i4b, 2_i4b
!
        select case( j )
!
            case( 1_i4b )
!
!               TEST 1 : SORT THE REAL DATA IN ASCENDING ORDER.
!
                call tri_insert( x  )
!
!               CHECK THAT THE SORTED ARRAY IS NON-DECREASING.
!
                i = count( x(1:n-1) > x(2:n) )
!
            case( 2_i4b )
!
!               TEST 2 : SORT THE INTEGER DATA IN ASCENDING ORDER.
!
                call tri_insert( y )
!
!               CHECK THAT THE SORTED ARRAY IS NON-DECREASING.
!
                i = count( y(1:n-1) > y(2:n) )
!
        end select
!
!       PRINT THE RESULT OF THE TESTS.
!
        if ( i==0_i4b ) then
!
            success = success + 1_i4b
!
!            write (prtunit,*) 'Test ',j,' is correct'
!
        else
!
            write (prtunit,*) 'Test ',j,' is incorrect'
            write (prtunit,*) '=> ',i,' > ',0_i4b
!
        end if
!
    end do
!
!   DEALLOCATE WORK ARRAYS.
!
    deallocate( x, y )
!
    if ( success/=2_i4b ) then
        write (prtunit,*) 2_i4b - success, 'tests are incorrect'
    end if
!
    write (prtunit,*) success, 'tests are correct'
!
!
! END OF PROGRAM test_tri_insert
! ==============================
!
end program test_tri_insert
